'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * Change.
 *
 * @type {Change}
 */

var Change = function () {

  /**
   * Constructor.
   *
   * @param {Editor} editor
   */

  function Change(editor) {
    var _this = this;

    _classCallCheck(this, Change);

    this.editor = editor;
    this.state = editor.getState();
    this.history = editor.getHistory();
    this.transforms = editor.getTransforms();
    this.operations = [];

    // Mix in a method for every transform type.
    Object.keys(this.transforms).forEach(function (key) {
      if (_this[key] != null) {
        throw new Error('Cannot overwrite change property "' + key + '".');
      }

      _this[key] = function () {
        var _transforms;

        for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
          args[_key] = arguments[_key];
        }

        return (_transforms = _this.transforms)[key].apply(_transforms, [_this].concat(args));
      };
    });
  }

  /**
   * Get the kind.
   *
   * @return {String}
   */

  _createClass(Change, [{
    key: 'apply',


    /**
     * Apply the change to the editor.
     *
     * @param {Object} options
     */

    value: function apply() {
      var options = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];
      var history = this.history;
      var state = this.state;
      var operations = this.operations;
      var _state = state;
      var cursorMarks = _state.cursorMarks;
      var selection = _state.selection;

      // If there are cursor marks and no mark operations have been performed,
      // then remove the cursor marks.

      if (state.cursorMarks) {
        var hasMark = operations.some(function (op) {
          return op.type == 'add-mark' || op.type == 'remove-mark';
        });

        if (!hasMark) state = state.merge({ cursorMarks: null });
      }

      // Determine whether we need to create a new snapshot.
      var snapshot = options.isDistinct == null ? this.shouldSnapshot() : options.isDistinct;

      // If we should, save a snapshot into the history before transforming.
      if (snapshot) history.save(state);

      // Apply the "isNative" flag, which is used to allow for natively-handled
      // content changes to skip rerendering the editor for performance.
      state = state.merge({
        isNative: !!options.isNative
      });

      return state;
    }
  }, {
    key: 'kind',
    get: function get() {
      return 'change';
    }
  }]);

  return Change;
}();

/**
 * Export.
 *
 * @type {Change}
 */

exports.default = Change;