'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * History.
 *
 * @type {History}
 */

var History = function () {

  /**
   * Constructor.
   *
   * @param {Object} properties
   */

  function History() {
    var properties = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

    _classCallCheck(this, History);

    var undos = properties.undos;
    var redos = properties.redos;
    var maxSize = properties.maxSize;

    this.undos = undos || [];
    this.redos = redos || [];
    this.maxSize = maxSize == null ? 100 : maxSize;
  }

  /**
   * Get the kind.
   *
   * @return {String}
   */

  _createClass(History, [{
    key: 'save',


    /**
     * Save a new `state` to the history.
     *
     * @param {State} state
     * @return {History}
     */

    value: function save(state) {
      this.undos.push(state);
      if (this.undos.length > this.maxSize) this.undos.unshift();
      this.redos = [];
      return this;
    }

    /**
     * Undo to the previous state in the history, adding the `current` state to
     * the redos stack.
     *
     * @param {State} current
     * @return {State}
     */

  }, {
    key: 'undo',
    value: function undo(current) {
      var previous = this.undos.pop();
      if (!previous) return current;
      this.redos.push(current);
      return previous;
    }

    /**
     * Redo to the next state in the history, adding the `current` state to the
     * undos stack.
     *
     * @param {State} current
     * @return {State}
     */

  }, {
    key: 'redo',
    value: function redo(current) {
      var next = this.redos.pop();
      if (!next) return current;
      this.undos.push(current);
      return next;
    }
  }, {
    key: 'kind',
    get: function get() {
      return 'history';
    }

    /**
     * Are there undoable snapshots?
     *
     * @return {Boolean}
     */

  }, {
    key: 'hasUndos',
    get: function get() {
      return this.undos.length > 0;
    }

    /**
     * Are there redoable snapshots?
     *
     * @return {Boolean}
     */

  }, {
    key: 'hasRedos',
    get: function get() {
      return this.redos.length > 0;
    }

    /**
     * Get the previous history state.
     *
     * @return {State}
     */

  }, {
    key: 'previous',
    get: function get() {
      return this.undos[this.undos.length - 1];
    }

    /**
     * Get the next history state.
     *
     * @return {State}
     */

  }, {
    key: 'next',
    get: function get() {
      return this.redos[this.redos.length - 1];
    }
  }]);

  return History;
}();

/**
 * Export.
 *
 * @type {History}
 */

exports.default = History;