'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.blur = blur;
exports.collapseToAnchor = collapseToAnchor;
exports.collapseToFocus = collapseToFocus;
exports.collapseToEndOf = collapseToEndOf;
exports.collapseToEndOfNextBlock = collapseToEndOfNextBlock;
exports.collapseToEndOfNextText = collapseToEndOfNextText;
exports.collapseToEndOfPreviousBlock = collapseToEndOfPreviousBlock;
exports.collapseToEndOfPreviousText = collapseToEndOfPreviousText;
exports.collapseToStartOf = collapseToStartOf;
exports.collapseToStartOfNextBlock = collapseToStartOfNextBlock;
exports.collapseToStartOfNextText = collapseToStartOfNextText;
exports.collapseToStartOfPreviousBlock = collapseToStartOfPreviousBlock;
exports.collapseToStartOfPreviousText = collapseToStartOfPreviousText;
exports.extendBackward = extendBackward;
exports.extendForward = extendForward;
exports.extendToEndOf = extendToEndOf;
exports.extendToStartOf = extendToStartOf;
exports.focus = focus;
exports.moveBackward = moveBackward;
exports.moveForward = moveForward;
exports.moveTo = moveTo;
exports.moveToOffsets = moveToOffsets;
exports.moveToRangeOf = moveToRangeOf;

var _selection11 = require('../models/selection');

var _selection12 = _interopRequireDefault(_selection11);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Blur the selection.
 *
 * @param {Transform} transform
 * @return {Transform}
 */

function blur(transform) {
  var state = transform.state;
  var _state = state;
  var document = _state.document;
  var selection = _state.selection;

  selection = selection.blur();
  selection = selection.normalize(document);
  state = state.merge({ selection: selection });
  transform.state = state;
  return transform;
}

/**
 * Move the focus point to the anchor point.
 *
 * @return {Transform}
 */

function collapseToAnchor(transform) {
  var state = transform.state;
  var _state2 = state;
  var document = _state2.document;
  var selection = _state2.selection;

  selection = selection.collapseToAnchor();
  selection = selection.normalize(document);
  state = state.merge({ selection: selection });
  transform.state = state;
  return transform;
}

/**
 * Move the anchor point to the focus point.
 *
 * @return {Transform}
 */

function collapseToFocus(transform) {
  var state = transform.state;
  var _state3 = state;
  var document = _state3.document;
  var selection = _state3.selection;

  selection = selection.collapseToFocus();
  selection = selection.normalize(document);
  state = state.merge({ selection: selection });
  transform.state = state;
  return transform;
}

/**
 * Move to the end of a `node`.
 *
 * @return {Transform}
 */

function collapseToEndOf(transform) {
  var _selection;

  var state = transform.state;
  var _state4 = state;
  var document = _state4.document;
  var selection = _state4.selection;

  for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    args[_key - 1] = arguments[_key];
  }

  selection = (_selection = selection).collapseToEndOf.apply(_selection, args);
  selection = selection.normalize(document);
  state = state.merge({ selection: selection });
  transform.state = state;
  return transform;
}

/**
 * Move the selection to the end of the next block.
 *
 * @param {State} state
 * @return {State}
 */

function collapseToEndOfNextBlock(transform) {
  var state = transform.state;
  var _state5 = state;
  var document = _state5.document;
  var selection = _state5.selection;

  var blocks = document.getBlocksAtRange(selection);
  var block = blocks.last();
  if (!block) return transform;

  var next = document.getNextBlock(block);
  if (!next) return transform;

  selection = selection.collapseToEndOf(next);
  selection = selection.normalize(document);
  state = state.merge({ selection: selection });
  transform.state = state;
  return transform;
}

/**
 * Move the selection to the end of the next text.
 *
 * @param {State} state
 * @return {State}
 */

function collapseToEndOfNextText(transform) {
  var state = transform.state;
  var _state6 = state;
  var document = _state6.document;
  var selection = _state6.selection;

  var texts = document.getTextsAtRange(selection);
  var text = texts.last();
  if (!text) return transform;

  var next = document.getNextText(text);
  if (!next) return transform;

  selection = selection.collapseToEndOf(next);
  selection = selection.normalize(document);
  state = state.merge({ selection: selection });
  transform.state = state;
  return transform;
}

/**
 * Move the selection to the end of the previous block.
 *
 * @param {State} state
 * @return {State}
 */

function collapseToEndOfPreviousBlock(transform) {
  var state = transform.state;
  var _state7 = state;
  var document = _state7.document;
  var selection = _state7.selection;

  var blocks = document.getBlocksAtRange(selection);
  var block = blocks.first();
  if (!block) return transform;

  var previous = document.getPreviousBlock(block);
  if (!previous) return transform;

  selection = selection.collapseToEndOf(previous);
  selection = selection.normalize(document);
  state = state.merge({ selection: selection });
  transform.state = state;
  return transform;
}

/**
 * Move the selection to the end of the previous text.
 *
 * @param {State} state
 * @return {State}
 */

function collapseToEndOfPreviousText(transform) {
  var state = transform.state;
  var _state8 = state;
  var document = _state8.document;
  var selection = _state8.selection;

  var texts = document.getTextsAtRange(selection);
  var text = texts.first();
  if (!text) return transform;

  var previous = document.getPreviousText(text);
  if (!previous) return transform;

  selection = selection.collapseToEndOf(previous);
  selection = selection.normalize(document);
  state = state.merge({ selection: selection });
  transform.state = state;
  return transform;
}

/**
 * Move to the start of a `node`.
 *
 * @return {Transform}
 */

function collapseToStartOf(transform, node) {
  var state = transform.state;
  var _state9 = state;
  var document = _state9.document;
  var selection = _state9.selection;

  selection = selection.collapseToStartOf(node);
  selection = selection.normalize(document);
  state = state.merge({ selection: selection });
  transform.state = state;
  return transform;
}

/**
 * Move the selection to the start of the next block.
 *
 * @param {State} state
 * @return {State}
 */

function collapseToStartOfNextBlock(transform) {
  var state = transform.state;
  var _state10 = state;
  var document = _state10.document;
  var selection = _state10.selection;

  var blocks = document.getBlocksAtRange(selection);
  var block = blocks.last();
  if (!block) return transform;

  var next = document.getNextBlock(block);
  if (!next) return transform;

  selection = selection.collapseToStartOf(next);
  selection = selection.normalize(document);
  state = state.merge({ selection: selection });
  transform.state = state;
  return transform;
}

/**
 * Move the selection to the start of the next text.
 *
 * @param {State} state
 * @return {State}
 */

function collapseToStartOfNextText(transform) {
  var state = transform.state;
  var _state11 = state;
  var document = _state11.document;
  var selection = _state11.selection;

  var texts = document.getTextsAtRange(selection);
  var text = texts.last();
  if (!text) return transform;

  var next = document.getNextText(text);
  if (!next) return transform;

  selection = selection.collapseToStartOf(next);
  selection = selection.normalize(document);
  state = state.merge({ selection: selection });
  transform.state = state;
  return transform;
}

/**
 * Move the selection to the start of the previous block.
 *
 * @param {State} state
 * @return {State}
 */

function collapseToStartOfPreviousBlock(transform) {
  var state = transform.state;
  var _state12 = state;
  var document = _state12.document;
  var selection = _state12.selection;

  var blocks = document.getBlocksAtRange(selection);
  var block = blocks.first();
  if (!block) return transform;

  var previous = document.getPreviousBlock(block);
  if (!previous) return transform;

  selection = selection.collapseToStartOf(previous);
  selection = selection.normalize(document);
  state = state.merge({ selection: selection });
  transform.state = state;
  return transform;
}

/**
 * Move the selection to the start of the previous text.
 *
 * @param {State} state
 * @return {State}
 */

function collapseToStartOfPreviousText(transform) {
  var state = transform.state;
  var _state13 = state;
  var document = _state13.document;
  var selection = _state13.selection;

  var texts = document.getTextsAtRange(selection);
  var text = texts.first();
  if (!text) return transform;

  var previous = document.getPreviousText(text);
  if (!previous) return transform;

  selection = selection.collapseToStartOf(previous);
  selection = selection.normalize(document);
  state = state.merge({ selection: selection });
  transform.state = state;
  return transform;
}

/**
 * Extend the focus point backward `n` characters.
 *
 * @param {Number} n (optional)
 * @return {Transform}
 */

function extendBackward(transform) {
  var _selection2;

  var state = transform.state;
  var _state14 = state;
  var document = _state14.document;
  var selection = _state14.selection;

  for (var _len2 = arguments.length, args = Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
    args[_key2 - 1] = arguments[_key2];
  }

  selection = (_selection2 = selection).extendBackward.apply(_selection2, args);
  selection = selection.normalize(document);
  state = state.merge({ selection: selection });
  transform.state = state;
  return transform;
}

/**
 * Extend the focus point forward `n` characters.
 *
 * @param {Number} n (optional)
 * @return {Transform}
 */

function extendForward(transform) {
  var _selection3;

  var state = transform.state;
  var _state15 = state;
  var document = _state15.document;
  var selection = _state15.selection;

  for (var _len3 = arguments.length, args = Array(_len3 > 1 ? _len3 - 1 : 0), _key3 = 1; _key3 < _len3; _key3++) {
    args[_key3 - 1] = arguments[_key3];
  }

  selection = (_selection3 = selection).extendForward.apply(_selection3, args);
  selection = selection.normalize(document);
  state = state.merge({ selection: selection });
  transform.state = state;
  return transform;
}

/**
 * Extend the focus point to the end of a `node`.
 *
 * @param {Node} node
 * @return {Transform}
 */

function extendToEndOf(transform) {
  var _selection4;

  var state = transform.state;
  var _state16 = state;
  var document = _state16.document;
  var selection = _state16.selection;

  for (var _len4 = arguments.length, args = Array(_len4 > 1 ? _len4 - 1 : 0), _key4 = 1; _key4 < _len4; _key4++) {
    args[_key4 - 1] = arguments[_key4];
  }

  selection = (_selection4 = selection).extendToEndOf.apply(_selection4, args);
  selection = selection.normalize(document);
  state = state.merge({ selection: selection });
  transform.state = state;
  return transform;
}

/**
 * Extend the focus point to the start of a `node`.
 *
 * @param {Node} node
 * @return {Transform}
 */

function extendToStartOf(transform) {
  var _selection5;

  var state = transform.state;
  var _state17 = state;
  var document = _state17.document;
  var selection = _state17.selection;

  for (var _len5 = arguments.length, args = Array(_len5 > 1 ? _len5 - 1 : 0), _key5 = 1; _key5 < _len5; _key5++) {
    args[_key5 - 1] = arguments[_key5];
  }

  selection = (_selection5 = selection).extendToStartOf.apply(_selection5, args);
  selection = selection.normalize(document);
  state = state.merge({ selection: selection });
  transform.state = state;
  return transform;
}

/**
 * Focus the selection.
 *
 * @return {Transform}
 */

function focus(transform) {
  var _selection6;

  var state = transform.state;
  var _state18 = state;
  var document = _state18.document;
  var selection = _state18.selection;

  for (var _len6 = arguments.length, args = Array(_len6 > 1 ? _len6 - 1 : 0), _key6 = 1; _key6 < _len6; _key6++) {
    args[_key6 - 1] = arguments[_key6];
  }

  selection = (_selection6 = selection).focus.apply(_selection6, args);
  selection = selection.normalize(document);
  state = state.merge({ selection: selection });
  transform.state = state;
  return transform;
}

/**
 * Move the selection backward `n` characters.
 *
 * @param {Number} n (optional)
 * @return {Transform}
 */

function moveBackward(transform) {
  var _selection7;

  var state = transform.state;
  var _state19 = state;
  var document = _state19.document;
  var selection = _state19.selection;

  for (var _len7 = arguments.length, args = Array(_len7 > 1 ? _len7 - 1 : 0), _key7 = 1; _key7 < _len7; _key7++) {
    args[_key7 - 1] = arguments[_key7];
  }

  selection = (_selection7 = selection).moveBackward.apply(_selection7, args);
  selection = selection.normalize(document);
  state = state.merge({ selection: selection });
  transform.state = state;
  return transform;
}

/**
 * Move the selection forward `n` characters.
 *
 * @param {Number} n (optional)
 * @return {Transform}
 */

function moveForward(transform) {
  var _selection8;

  var state = transform.state;
  var _state20 = state;
  var document = _state20.document;
  var selection = _state20.selection;

  for (var _len8 = arguments.length, args = Array(_len8 > 1 ? _len8 - 1 : 0), _key8 = 1; _key8 < _len8; _key8++) {
    args[_key8 - 1] = arguments[_key8];
  }

  selection = (_selection8 = selection).moveForward.apply(_selection8, args);
  selection = selection.normalize(document);
  state = state.merge({ selection: selection });
  transform.state = state;
  return transform;
}

/**
 * Move the selection to a specific anchor and focus point.
 *
 * @param {State} state
 * @param {Object} properties
 * @return {State}
 */

function moveTo(transform, properties) {
  var state = transform.state;
  var _state21 = state;
  var document = _state21.document;
  var selection = _state21.selection;

  // Allow for passing a `Selection` object.

  if (properties instanceof _selection12.default) {
    properties = {
      anchorKey: properties.anchorKey,
      anchorOffset: properties.anchorOffset,
      focusKey: properties.focusKey,
      focusOffset: properties.focusOffset,
      isFocused: properties.isFocused
    };
  }

  // Pass in properties, and force `isBackward` to be re-resolved.
  selection = selection.merge(_extends({}, properties, {
    isBackward: null
  }));

  selection = selection.normalize(document);
  state = state.merge({ selection: selection });
  transform.state = state;
  return transform;
}

/**
 * Move the selection to `anchor` and `focus` offsets.
 *
 * @param {Number} anchor
 * @param {Number} focus (optional)
 * @return {Transform}
 */

function moveToOffsets(transform) {
  var _selection9;

  var state = transform.state;
  var _state22 = state;
  var document = _state22.document;
  var selection = _state22.selection;

  for (var _len9 = arguments.length, args = Array(_len9 > 1 ? _len9 - 1 : 0), _key9 = 1; _key9 < _len9; _key9++) {
    args[_key9 - 1] = arguments[_key9];
  }

  selection = (_selection9 = selection).moveToOffsets.apply(_selection9, args);
  selection = selection.normalize(document);
  state = state.merge({ selection: selection });
  transform.state = state;
  return transform;
}

/**
 * Move to the entire range of `start` and `end` nodes.
 *
 * @param {Node} start
 * @param {Node} end (optional)
 * @return {Transform}
 */

function moveToRangeOf(transform) {
  var _selection10;

  var state = transform.state;
  var _state23 = state;
  var document = _state23.document;
  var selection = _state23.selection;

  for (var _len10 = arguments.length, args = Array(_len10 > 1 ? _len10 - 1 : 0), _key10 = 1; _key10 < _len10; _key10++) {
    args[_key10 - 1] = arguments[_key10];
  }

  selection = (_selection10 = selection).moveToRangeOf.apply(_selection10, args);
  selection = selection.normalize(document);
  state = state.merge({ selection: selection });
  transform.state = state;
  return transform;
}