"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.insertTextOperation = insertTextOperation;
exports.removeTextOperation = removeTextOperation;
exports.addMarkOperation = addMarkOperation;
exports.removeMarkOperation = removeMarkOperation;

/**
 * Insert `text` at a `path` and `offset`.
 *
 * @param {State} state
 * @param {Array} path
 * @param {Number} offset
 * @param {String} string
 * @return {State}
 */

function insertTextOperation(state, path, offset, string) {
  var _state = state;
  var document = _state.document;

  var node = document.assertPath(path);
  node = node.insertText(offset, string);
  document = document.updateDescendant(node);
  state = state.merge({ document: document });
  return state;
}

/**
 * Remove `text` at a `path` and `offset`.
 *
 * @param {State} state
 * @param {Array} path
 * @param {Number} offset
 * @param {Number} length
 * @return {State}
 */

function removeTextOperation(state, path, offset, length) {
  var _state2 = state;
  var document = _state2.document;

  var node = document.assertPath(path);
  node = node.removeText(offset, offset + length);
  document = document.updateDescendant(node);
  state = state.merge({ document: document });
  return state;
}

/**
 * Add a `mark` at a `path` and `offset` and `length`.
 *
 * @param {State} state
 * @param {Array} path
 * @param {Number} offset
 * @param {Number} length
 * @param {Mark} mark
 * @return {State}
 */

function addMarkOperation(state, path, offset, length, mark) {
  var _state3 = state;
  var document = _state3.document;

  var node = document.assertPath(path);
  node = node.addMark(offset, offset + length, mark);
  document = document.updateDescendant(node);
  state = state.merge({ document: document });
  return state;
}

/**
 * Remove a `mark` from a `path` and `offset` and `length`.
 *
 * @param {State} state
 * @param {Array} path
 * @param {Number} offset
 * @param {Number} length
 * @param {Mark} mark
 * @return {State}
 */

function removeMarkOperation(state, path, offset, length, mark) {
  var _state4 = state;
  var document = _state4.document;

  var node = document.assertPath(path);
  node = node.addMark(offset, offset + length, mark);
  document = document.updateDescendant(node);
  state = state.merge({ document: document });
  return state;
}